package cn.com.qmth.stmms.biz.common.domain.card;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

/**
 * 卡格式文件内容结构
 */
@JsonIgnoreProperties(ignoreUnknown = true)
public class CardFile {

    private String version;

    private boolean adapted;

    private List<CardPageWrapper> pages;

    @JsonIgnore
    private SliceConfig sliceConfig;

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public boolean isAdapted() {
        return adapted;
    }

    public void setAdapted(boolean adapted) {
        this.adapted = adapted;
    }

    public List<CardPageWrapper> getPages() {
        return pages;
    }

    public void setPages(List<CardPageWrapper> pages) {
        this.pages = pages;
    }

    /**
     * 解析卡格式文件中的裁切图坐标，用于云阅卷同步
     *
     * @return
     */
    public SliceConfig getSliceConfig() {
        if (sliceConfig == null) {
            sliceConfig = new SliceConfig();
            if (pages != null) {
                int pageNumber = 0;
                for (CardPageWrapper pageWrapper : pages) {
                    pageNumber++;
                    //默认使用遮盖模式
                    sliceConfig.addConfig(pageNumber, 0, 0, 0, 0);
                }
            }
        }
        return sliceConfig;
    }

    /**
     * 校验字段并设置初始值
     */
    private void validate() {
        version = "1.0.0";
        adapted = false;
        for (CardPageWrapper cpw : pages) {
            cpw.getExchange().validate();
        }
    }

    /**
     * 输出为字节数组，用于保存到文件或其他输出流
     *
     * @return
     * @throws JsonProcessingException
     */
    public byte[] output() throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
        return mapper.writeValueAsBytes(this);
    }

    /**
     * 从输入流解析卡格式文件
     *
     * @param ins
     * @return
     * @throws IOException
     */
    public static CardFile parse(InputStream ins) throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        CardFile obj = mapper.readValue(ins, CardFile.class);
        obj.validate();
        return obj;
    }
}
